# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['FileConfDriver']

"""This module specifies file drivers.

They work relatively to a conf directory which is given by the environment
variable ``B3J0F_CONF_DIR`` or if not present, by the relative path ``~/etc``.
"""

from stat import ST_SIZE

from os import stat, environ

from os.path import exists, join, expanduser

from b3j0f.conf.driver.core import ConfDriver


class FileConfDriver(ConfDriver):
    """Conf Manager dedicated to files.
    """

    CONF_DIR_VAR = 'B3J0F_CONF_DIR'  #: conf dir environment variable
    #: final conf dir, given by CONF_DIR_VAR environment variable or '~/etc'
    CONF_DIR = environ.get(
        CONF_DIR_VAR, join(expanduser('~'), 'etc')
    )

    def exists(self, conf_path, *args, **kwargs):

        path = FileConfDriver.get_path(conf_path)

        result = exists(path) and stat(path)[ST_SIZE]

        return result

    @staticmethod
    def get_path(conf_path):

        result = join(FileConfDriver.CONF_DIR, conf_path)

        return result



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['JSONConfDriver']


try:
    from json import loads, dump
except ImportError:
    from simplejson import loads, dump

from b3j0f.conf.driver.file.core import FileConfDriver


class JSONConfDriver(FileConfDriver):
    """Manage json configuration.
    """

    __register__ = True  #: Register it automatically among global managers.

    def _has_category(
        self, conf_resource, category, logger, *args, **kwargs
    ):
        return category.name in conf_resource

    def _has_parameter(
        self, conf_resource, category, param, logger,
        *args, **kwargs
    ):
        return param.name in conf_resource[category.name]

    def _get_conf_resource(
        self, logger, conf_path=None, *args, **kwargs
    ):
        result = {}

        if conf_path is not None:
            result = None

            path = FileConfDriver.get_path(conf_path)

            try:
                with open(path, 'r') as handle:
                    content = handle.read()
                    result = loads(content)

            except Exception:
                pass

        return result

    def _get_categories(self, conf_resource, logger, *args, **kwargs):
        return conf_resource.keys()

    def _get_parameters(
        self, conf_resource, category, logger, *args, **kwargs
    ):
        return conf_resource[category.name].keys()

    def _get_value(
        self, conf_resource, category, param, *args, **kwargs
    ):
        return conf_resource[category.name][param.name]

    def _set_category(
        self, conf_resource, category, logger, *args, **kwargs
    ):
        conf_resource.setdefault(category.name, {})

    def _set_parameter(
        self, conf_resource, category, param, logger,
        *args, **kwargs
    ):
        conf_resource[category.name][param.name] = param.value

    def _update_conf_resource(
        self, conf_resource, conf_path, *args, **kwargs
    ):

        path = FileConfDriver.get_path(conf_path)

        try:
            with open(path, 'w') as handle:
                dump(conf_resource, handle)

        except Exception:
            pass



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = [
    'add_configurable', 'remove_configurable',
    'on_update_conf_file',
    'Watcher',
    'get_watcher', 'start_watch', 'stop_watch', 'change_sleeping_time'
]


try:
    from threading import Timer
except ImportError:
    from dummythreading import Timer

from stat import ST_MTIME

from os import stat
from os.path import exists, expanduser

from b3j0f.conf.params import Configuration, Parameter, Category
from b3j0f.conf.configurable.core import Configurable

#: Dictionary of (mtime, configurables) by configuration file.
_CONFIGURABLES_BY_CONF_FILES = {}
_MTIME_BY_CONF_FILES = {}


def add_configurable(configurable):
    """Add input configurable to list of watchers.

    :param Configurable configurable: configurable to add.
    """

    for conf_file in configurable.conf_files:

        conf_file = expanduser(conf_file)

        configurables = _CONFIGURABLES_BY_CONF_FILES.setdefault(
            conf_file, set())

        configurables.add(configurable)


def remove_configurable(configurable):
    """Remove configurable to list of watchers.

    :param Configurable configurable: configurable to remove.
    """

    for conf_file in configurable.conf_files:

        conf_file = expanduser(conf_file)

        configurables = _CONFIGURABLES_BY_CONF_FILES.get(conf_file)

        if configurables:
            configurables.remove(configurable)

            if not configurables:
                del _CONFIGURABLES_BY_CONF_FILES[conf_file]


def on_update_conf_file(conf_file):
    """Apply configuration on all configurables which watch input
    conf_file.

    :param str conf_file: configuration file to reconfigure.
    """

    conf_file = expanduser(conf_file)

    configurables = _CONFIGURABLES_BY_CONF_FILES.get(conf_file)

    if configurables:

        for configurable in configurables:

            configurable.apply_configuration(conf_file=conf_file)

# default value for sleeping_time
DEFAULT_SLEEPING_TIME = 5


class Watcher(Configurable):
    """Watches all sleeping_time.
    """

    CONF_FILE = 'configuration/watcher.conf'

    CATEGORY = 'WATCHER'
    SLEEPING_TIME = 'sleeping_time'

    DEFAULT_CONFIGURATION = Configuration(
        Category(
            CATEGORY,
            Parameter(SLEEPING_TIME, value=DEFAULT_SLEEPING_TIME, parser=int)
        )
    )

    def __init__(self, sleeping_time=DEFAULT_SLEEPING_TIME, *args, **kwargs):

        super(Watcher, self).__init__(*args, **kwargs)

        self._sleeping_time = sleeping_time
        self._timer = None

    @property
    def sleeping_time(self):
        return self._sleeping_time

    @sleeping_time.setter
    def sleeping_time(self, value):
        """Change value of sleeping_time.
        """

        self._sleeping_time = value
        # restart the timer
        self.stop()
        self.start()

    def _get_conf_files(self, *args, **kwargs):

        result = super(Watcher, self)._get_conf_files(*args, **kwargs)

        result.append(Watcher.CONF_FILE)

        return result

    def _conf(self, *args, **kwargs):

        result = super(Watcher, self)._conf(*args, **kwargs)

        result.add_unified_category(
            name=Watcher.CATEGORY,
            new_content=(
                Parameter(Watcher.SLEEPING_TIME, parser=int)))

        return result

    def run(self):
        global _CONFIGURABLES_BY_CONF_FILES

        for conf_file in _CONFIGURABLES_BY_CONF_FILES:

            # check file exists
            if exists(conf_file):
                # get mtime
                mtime = stat(conf_file)[ST_MTIME]
                # compare with old mtime
                old_mtime = _MTIME_BY_CONF_FILES.setdefault(conf_file, mtime)
                if old_mtime != mtime:
                    on_update_conf_file(conf_file)

    def _run(self):
        self._timer = Timer(self.sleeping_time, self._run)
        self._timer.start()
        self.run()

    def start(self):
        self._timer = Timer(self.sleeping_time, self._run)
        self._timer.start()

    def stop(self):
        if self._timer is not None:
            self._timer.cancel()


def get_watcher():
    """Get global watcher.

    :return: global watcher.
    :rtype: Watcher
    """

    global _WATCHER  # try to load a global watcher

    try:
        result = _WATCHER
    except NameError:  # init a new watcher if _WATCHER is undefined
        result = _WATCHER = Watcher()

    return result


def start_watch():
    """Start default watcher.
    """

    watcher = get_watcher()
    watcher.start()


def stop_watch():
    """Stop default watcher.
    """

    watcher = get_watcher()
    watcher.stop()


def change_sleeping_time(sleeping_time):
    """Change of sleeping_time for default watcher.
    """

    watcher = get_watcher()
    watcher.sleeping_time = sleeping_time



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['MetaConfDriver', 'ConfDriver']


from b3j0f.utils.path import lookup, getpath

from b3j0f.conf.params import Configuration, Parameter, Category, ParamList


class MetaConfDriver(type):
    """ConfDriver meta class which register all driver in a global
    set of drivers.
    """

    def __init__(self, name, bases, attrs):

        super(MetaConfDriver, self).__init__(name, bases, attrs)

        # if the class claims to be registered
        if self.__register__:
            # add it among drivers
            ConfDriver._MANAGERS[getpath(self)] = self


class ConfDriver(object):
    """Base class for managing conf.
    """

    """Apply meta class for registering automatically it among global drivers
    if __register__ is True
    """
    __metaclass__ = MetaConfDriver

    """Static param which allows this class to be automatically registered
    among drivers.
    """
    __register__ = False

    CONF_FILE = 'CONF_FILE'

    _MANAGERS = {}  #: Private set of shared driver types.

    def handle(self, conf_path, logger):
        """True iif input conf_path can be handled by self.

        :return: True iif input conf_path can be handled by self.
        :rtype: bool
        """

        conf_resource = self._get_conf_resource(
            conf_path=conf_path, logger=logger)

        result = conf_resource is not None

        return result

    def exists(self, conf_path):
        """True if conf_path exist related to this driver behaviour.
        """

        raise NotImplementedError()

    def get_conf(
        self, conf_path, logger, conf=None, override=True
    ):
        """Parse a configuration_files with input conf and returns
        parameters and errors by param name.

        :param str conf_path: conf file to parse and from get parameters.
        :param Configuration conf: conf to fill with conf_path values and
            conf param names.
        :param Logger logger: logger to use in order to trace
            information/error.
        :param bool override: if True (by default), override self
            configuration.
        """

        conf_resource = None

        result = None
        # ensure conf_path exists and is not empty.
        if self.exists(conf_path):
            try:
                # first, read conf file
                conf_resource = self._get_conf_resource(
                    conf_path=conf_path,
                    logger=logger
                )

            except Exception as e:
                # if an error occured, log it
                logger.error(
                    'Impossible to parse conf_path {0} with {1}: {2}'.format(
                        conf_path,
                        type(self),
                        e
                    )
                )

            else:  # else process conf file
                if conf_resource is None:
                    return result

                result = Configuration() if conf is None else conf

                categories = self._get_categories(
                    conf_resource=conf_resource,
                    logger=logger
                )

                for category_name in categories:
                    # do something only for referenced categories
                    if category_name in result:
                        category = result.setdefault(
                            category_name,
                            Category(category_name)
                        )

                        if isinstance(category, Category):
                            parameters = self._get_parameters(
                                conf_resource=conf_resource,
                                category=category,
                                logger=logger
                            )

                            for name in parameters:
                                # if param name exists in conf
                                if name in category:
                                    # copy parameter
                                    param = category[name].copy()

                                # else create not local parameter
                                else:
                                    param = Parameter(name, local=False)

                                param = category.setdefault(name, param)

                                value = self._get_value(
                                    conf_resource=conf_resource,
                                    category=category,
                                    param=param,
                                    logger=logger
                                )

                                if value not in (None, ''):
                                    if override or param.value in (None, ''):
                                        param.value = value

                        elif isinstance(category, ParamList):
                            paramlist = category
                            category = Category(category_name)
                            result.categories[category_name] = category

                            parameters = self._get_parameters(
                                conf_resource=conf_resource,
                                category=category,
                                logger=logger
                            )

                            for name in parameters:
                                param = Parameter(
                                    name,
                                    local=False,
                                    parser=paramlist.parser,
                                    asitem=category
                                )

                                param = category.setdefault(name, param)

                                value = self._get_value(
                                    conf_resource=conf_resource,
                                    category=category,
                                    param=param,
                                    logger=logger
                                )

                                if value not in (None, ''):
                                    if override or param.value in (None, ''):
                                        param.value = value

        return result

    def set_conf(self, conf_path, conf, logger):
        """Set input conf in input conf_path.

        :param str conf_path: conf file to parse and from get parameters.
        :param Configuration conf: conf to write to conf_path.
        :param Logger logger: used to log info/errors
        """

        result = None
        conf_resource = None

        try:  # get conf_resource
            conf_resource = self._get_conf_resource(
                conf_path=conf_path,
                logger=logger)

        except Exception as e:
            # if an error occured, stop processing
            logger.error(
                'Impossible to parse conf_path {0}: {1}'.format(
                    conf_path, e))
            result = e

        # if conf_path can not be loaded, get default config conf_resource
        if conf_resource is None:
            conf_resource = self._get_conf_resource(logger=logger)

        # iterate on all conf items
        for category in conf:

            # set category
            self._set_category(
                conf_resource=conf_resource, category=category,
                logger=logger)

            # iterate on parameters
            for param in category:

                # set param
                self._set_parameter(
                    conf_resource=conf_resource,
                    category=category,
                    param=param,
                    logger=logger)

        # write conf_resource in conf file
        self._update_conf_resource(
            conf_resource=conf_resource, conf_path=conf_path)

        return result

    @staticmethod
    def get_drivers():
        """Get global defined drivers.
        """

        return set(ConfDriver._MANAGERS.values())

    @staticmethod
    def get_driver(path):
        """Add a conf driver by its path definition.

        :param str path: driver path to add. Must be a full path from a known
            package/module.
        """

        # try to get if from global definition
        result = ConfDriver._MANAGERS.get(path)

        # if not already added
        if result is None:
            # resolve it and add it in global definition
            result = lookup(path)
            ConfDriver._MANAGERS[path] = result

        return result

    def _get_categories(self, conf_resource, logger):
        """Get a list of category names in conf_resource.
        """

        raise NotImplementedError()

    def _get_parameters(self, conf_resource, category, logger):
        """Get a list of param names in conf_resource related to category.
        """

        raise NotImplementedError()

    def _has_category(self, conf_resource, category, logger):
        """True iif input conf_resource contains input category.
        """

        raise NotImplementedError()

    def _has_parameter(self, conf_resource, category, param, logger):
        """True iif input conf_resource has input parameter_name in input
            category.
        """

        raise NotImplementedError()

    def _get_conf_resource(self, logger, conf_path=None):
        """Get config conf_resource.

        :param str conf_path: if not None, the config conf_resource is
            conf_path content.
        :param Logger logger: logger used to log processing information
        :return: empty config conf_resource if conf_path is None, else
            conf_path content.
        """

        raise NotImplementedError()

    def _get_value(self, conf_resource, category, param):
        """Get a param related to input conf_resource, category and param
        """

        raise NotImplementedError()

    def _set_category(self, conf_resource, category, logger):
        """Set category on conf_resource.
        """

        raise NotImplementedError()

    def _set_parameter(self, conf_resource, category, param, logger):
        """Set param on conf_resource.
        """

        raise NotImplementedError()

    def _update_conf_resource(self, conf_resource, conf_path):
        """Write conf_resource into conf_path.
        """

        raise NotImplementedError()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------


from unittest import TestCase, main

from b3j0f.conf.configurable.core import Configurable
from b3j0f.conf.params import Configuration, Category, Parameter

from tempfile import NamedTemporaryFile

from os import remove


class ConfigurableTest(TestCase):

    def setUp(self):

        self.conf_paths = (
            NamedTemporaryFile().name,
            NamedTemporaryFile().name
        )

        self.configurable = Configurable()

        self.conf = Configuration(
            Category(
                'A',
                Parameter('a', value='a'),
                Parameter('2', value=2, parser=int),
                Parameter('error', value='error', parser=float)),
            Category(
                'B',
                Parameter('a', value='b'),
                Parameter('b', value='b')))

    def test_configuration_files(self):

        configurable = Configurable()
        configurable.conf_paths = self.conf_paths

        self.assertEqual(
            configurable.conf_paths,
            self.conf_paths)

        configurable = Configurable(
            conf_paths=self.conf_paths)

        self.assertEqual(
            configurable.conf_paths,
            self.conf_paths)

    def test_auto_conf(self):

        configurable = Configurable()

        self.assertTrue(configurable.auto_conf)

        configurable.auto_conf = False

        self.assertFalse(configurable.auto_conf)

    def test_logging_level(self):

        configurable = Configurable()

        self.assertTrue(configurable.log_lvl, 'INFO')

        configurable = Configurable(log_lvl='DEBUG')

        self.assertTrue(configurable.log_lvl, 'DEBUG')

        configurable.log_lvl = 'INFO'

        self.assertTrue(configurable.log_lvl, 'INFO')

    def test_configuration(self):

        # test to get from no file
        configurable = Configurable()

        conf = configurable.get_conf()

        self.assertEqual(len(conf), len(self.conf))

        # test to get from files which do not exist
        configurable.conf_paths = self.conf_paths

        for conf_path in self.conf_paths:
            try:
                remove(conf_path)
            except OSError:
                pass

        conf = configurable.get_conf()

        self.assertEqual(len(conf), len(self.conf))

        # get parameters from empty files
        for conf_path in self.conf_paths:
            with open(conf_path, 'w') as _file:
                _file.write('\n')

        conf = configurable.get_conf()

        self.assertEqual(len(conf), len(self.conf))

        # get parameters from empty files and empty parsing_rules
        conf = Configuration()
        configurable.get_conf(conf=conf)

        self.assertEqual(len(conf), 0)

        # fill files
        configurable = Configurable(
            conf_paths=self.conf_paths)

        # add first category in conf file[0]
        configurable.set_conf(
            conf_path=self.conf_paths[0],
            conf=Configuration(self.conf['A']),
            driver=configurable._drivers.split(',')[0])

        # add second category in conf file[1]
        configurable.set_conf(
            conf_path=self.conf_paths[1],
            conf=Configuration(self.conf['B']),
            driver=configurable._drivers.split(',')[1])

        conf = configurable.get_conf(conf=self.conf)

        unified_configuration = conf.unify()
        parameters = unified_configuration[Configuration.VALUES]
        errors = unified_configuration[Configuration.ERRORS]

        self.assertTrue('a' in parameters and 'a' not in errors)
        self.assertEqual(parameters['a'].value, 'b')
        self.assertTrue('2' in parameters and '2' not in errors)
        self.assertEqual(parameters['2'].value, 2)
        self.assertTrue('b' in parameters and 'b' not in errors)
        self.assertEqual(parameters['b'].value, 'b')
        self.assertTrue('error' in errors and 'error' not in parameters)

    def test_reconfigure(self):

        self.assertTrue(self.configurable.auto_conf)

        conf = Configuration(
            Category(
                'TEST',
                Parameter('auto_conf', value=False)))

        self.configurable.configure(conf=conf)
        self.assertFalse(self.configurable.auto_conf)
        self.assertEqual(self.configurable.log_lvl, 'INFO')

        conf = Configuration(
            Category(
                'TEST',
                Parameter('log_lvl', value='DEBUG')))

        self.configurable.configure(conf=conf)
        self.assertEqual(self.configurable.log_lvl, 'INFO')

        self.configurable.reconf_once = True
        self.configurable.configure(conf=conf)
        self.assertEqual(self.configurable.log_lvl, 'DEBUG')
        self.assertFalse(self.configurable.reconf_once)

        self.configurable.log_lvl = 'INFO'
        self.configurable.auto_conf = True
        self.configurable.configure(conf=conf)
        self.assertEqual(self.configurable.log_lvl, 'DEBUG')

    def test_parser_inheritance(self):

        class _Configurable(Configurable):

            def _conf(self, *args, **kwargs):

                result = super(_Configurable, self)._conf(
                    *args, **kwargs)

                result += Category('PLOP')

                return result

        configurable = Configurable()

        _configurable = _Configurable()

        self.assertEqual(
            len(configurable.conf) + 1,
            len(_configurable.conf))

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------


from unittest import main, TestCase

from b3j0f.conf.params import Parameter, Category
from b3j0f.conf.configurable.core import Configurable
from b3j0f.conf.configurable.decorator import conf_paths, add_category


class DecoratorTest(TestCase):
    """Configuration Manager unittest class.
    """

    def test_conf_paths(self):

        test_conf_paths = ["test1", "test2"]

        @conf_paths(*test_conf_paths)
        class TestConfigurable(Configurable):
            pass

        testConfigurable = TestConfigurable()

        configurable_conf_paths = testConfigurable.conf_paths

        for i in range(1, len(test_conf_paths)):
            self.assertEqual(test_conf_paths[-i], configurable_conf_paths[-i])

    def test_add_category(self):

        CATEGORY = 'TEST'

        @add_category(name=CATEGORY)
        class TestConfigurable(Configurable):
            pass

        tc = TestConfigurable()

        self.assertTrue(CATEGORY in tc.conf)
        self.assertTrue(len(tc.conf) > 0)
        self.assertTrue(len(tc.conf[CATEGORY]) > 0)

        category_len = len(tc.conf[CATEGORY])

        parameters = [Parameter('a'), Parameter('b')]

        @add_category(name=CATEGORY, content=parameters)
        class TestConfigurable(Configurable):
            pass

        tc = TestConfigurable()

        self.assertTrue(CATEGORY in tc.conf)
        self.assertTrue(len(tc.conf) > 0)
        self.assertEqual(
            len(tc.conf[CATEGORY]), category_len + len(parameters))

        @add_category(name=CATEGORY, unified=False)
        class TestConfigurable(Configurable):
            pass

        tc = TestConfigurable()

        self.assertTrue(CATEGORY in tc.conf)
        self.assertTrue(len(tc.conf) > 0)
        self.assertEqual(len(tc.conf[CATEGORY]), 0)

        @add_category(name=CATEGORY, unified=False, content=parameters)
        class TestConfigurable(Configurable):
            pass

        tc = TestConfigurable()

        self.assertTrue(CATEGORY in tc.conf)
        self.assertTrue(len(tc.conf) > 0)
        self.assertEqual(len(tc.conf[CATEGORY]), len(parameters))

        category = Category(CATEGORY, *parameters)

        @add_category(name=CATEGORY, unified=False, content=category)
        class TestConfigurable(Configurable):
            pass

        tc = TestConfigurable()

        self.assertTrue(CATEGORY in tc.conf)
        self.assertTrue(len(tc.conf) > 0)
        self.assertEqual(len(tc.conf[CATEGORY]), len(category))

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

from unittest import main

from tempfile import NamedTemporaryFile

from os import remove

from b3j0f.utils.ut import UTCase

from b3j0f.conf.configurable.core import Configurable
from b3j0f.conf.configurable.registry import ConfigurableRegistry
from b3j0f.conf.driver.file.core import FileConfDriver


class TestConfigurable(Configurable):
    pass


class TestRegistry(ConfigurableRegistry):

    def _get_conf_paths(self, *args, **kwargs):

        result = super(TestRegistry, self)._get_conf_paths(*args, **kwargs)
        result.append(NamedTemporaryFile().name)
        return result


class ManagerTest(UTCase):

    def setUp(self):
        pass

    def test_apply_configuration(self):

        driver = TestRegistry()

        conf_path = FileConfDriver.get_path(driver.conf_paths[-1])

        configurable_name = 'test'
        full_configurable_name = '{0}{1}'.format(
            configurable_name, ConfigurableRegistry.CONFIGURABLE_SUFFIX)
        configurable_type_name = '{0}{1}'.format(
            configurable_name, ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX)
        # ensure configurable doesn't exis
        self.assertFalse(configurable_name in driver)

        self.assertEqual(len(driver.configurables), 0)
        self.assertEqual(len(driver.configurable_types), 0)

        driver[configurable_name] = Configurable()

        self.assertTrue(driver[configurable_name].auto_conf)

        configurable_path = "b3j0f.conf.configurable.core.Configurable"

        with open(conf_path, 'w+') as conf_file:
            conf_file.write("[MANAGER]")
            # set configurable
            conf_file.write("\n{0}=".format(full_configurable_name))
            conf_file.write(configurable_path)
            # set configurable type
            conf_file.write("\n{0}=".format(configurable_type_name))
            conf_file.write(configurable_path)
            # set sub-configurable auto_conf to false
            configurable_category = \
                ConfigurableRegistry.get_configurable_category(
                    configurable_name)
            conf_file.write("\n[{0}]".format(configurable_category))
            conf_file.write("\nauto_conf=false")

        driver.apply_configuration()

        remove(conf_path)

        self.assertEqual(len(driver.configurables), 1)
        self.assertEqual(len(driver.configurable_types), 1)

        # check if configurable and short attribute exist
        self.assertFalse(driver[configurable_name].auto_conf)
        self.assertTrue(isinstance(driver[configurable_name], Configurable))

        # check if configurable type exist
        self.assertEqual(
            driver.configurable_types[configurable_name], Configurable)

        # change type with str or Configurable class
        driver.configurable_types[configurable_name] = configurable_path
        self.assertEqual(
            driver.configurable_types[configurable_name], Configurable)
        driver.configurable_types[configurable_name] = Configurable
        self.assertEqual(
            driver.configurable_types[configurable_name], Configurable)

        # change type in order to do not remove old value
        driver.test_configurable_type = Configurable
        self.assertTrue(isinstance(driver[configurable_name], Configurable))
        driver.test = configurable_path
        self.assertTrue(isinstance(driver[configurable_name], Configurable))
        driver.test = Configurable
        self.assertTrue(isinstance(driver[configurable_name], Configurable))
        driver.test = Configurable()
        self.assertTrue(isinstance(driver[configurable_name], Configurable))

        self.assertEqual(len(driver.configurables), 1)
        self.assertEqual(len(driver.configurable_types), 1)

        # change type value in order to remove old value
        driver.configurable_types[configurable_name] = TestConfigurable
        self.assertFalse(configurable_name in driver)
        self.assertEqual(
            driver.configurable_types[configurable_name], TestConfigurable)

        self.assertEqual(len(driver.configurables), 0)
        self.assertEqual(len(driver.configurable_types), 1)

        driver[configurable_name] = Configurable()
        self.assertFalse(configurable_name in driver)

        self.assertEqual(len(driver.configurables), 0)
        self.assertEqual(len(driver.configurable_types), 1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------


from logging import getLogger

from b3j0f.utils.ut import UTCase

from unittest import main

from b3j0f.conf.params import Configuration, Category, Parameter
from b3j0f.conf.driver.file.core import FileConfDriver

from pickle import loads, dump

from os import remove


class TestConfDriver(FileConfDriver):
    """Configuration Manager for test.
    """

    __register__ = True

    def _has_category(
        self, conf_resource, category, logger, *args, **kwargs
    ):

        return category in conf_resource

    def _has_parameter(
        self, conf_resource, category, param, logger,
        *args, **kwargs
    ):

        return param.name in conf_resource[category.name]

    def _get_categories(self, conf_resource, logger, *args, **kwargs):
        return conf_resource.keys()

    def _get_parameters(
        self, conf_resource, category, logger, *args, **kwargs
    ):
        return conf_resource[category.name].keys()

    def _get_conf_resource(
        self, logger, conf_path=None, *args, **kwargs
    ):

        result = dict()

        if conf_path is not None:

            with open(conf_path, 'rb') as handle:

                try:
                    result = loads(handle.read())

                except Exception:
                    pass

        return result

    def _get_value(
        self, conf_resource, category, param, logger,
        *args, **kwargs
    ):

        return conf_resource[category.name][param.name]

    def _set_category(
        self, conf_resource, category, logger, *args, **kwargs
    ):

        conf_resource.setdefault(category.name, dict())

    def _set_parameter(
        self, conf_resource, category, param, logger,
        *args, **kwargs
    ):
        conf_resource[category.name][param.name] = param.value

    def _update_conf_resource(
        self, conf_resource, conf_path, *args, **kwargs
    ):

        with open(conf_path, 'wb') as handle:

            try:
                dump(conf_resource, handle)

            except Exception:
                pass


class FileConfDriverTest(UTCase):
    """Configuration Manager unittest class.
    """

    ERROR_PARAMETER = 'foo4'

    def setUp(self):
        self.logger = getLogger()

        self.manager = self._get_conf_manager()

        self.conf = Configuration(
            Category(
                'A',
                Parameter('a', value=0, parser=int),  # a is 0
                Parameter('b', value=True, parser=Parameter.bool)
            ),  # b is overriden
            Category(
                'B',
                Parameter('b', value=1, parser=int),  # b is 1
                Parameter('c', value='er', parser=int)   # error
            )
        )

        self.conf_path = self.get_conf_file()

    def get_conf_file(self):

        return '/tmp/b3j0f.conf'

    def _remove(self):
        try:
            remove(self.conf_path)
        except OSError:
            pass

    def _open(self):

        try:
            open(self.conf_path, 'wb').close()
        except OSError:
            pass

    def test_configuration(self):

        # try to get conf from not existing file
        self._remove()

        conf = self.manager.get_conf(
            conf_path=self.conf_path,
            logger=self.logger
        )

        self.assertIsNone(conf)

        # get conf from an empty media
        self._open()

        conf = self.manager.get_conf(
            conf_path=self.conf_path,
            logger=self.logger
        )

        self.assertIsNone(conf)

        # get full conf
        self.manager.set_conf(
            conf_path=self.conf_path,
            conf=self.conf,
            logger=self.logger
        )

        conf = self.manager.get_conf(
            conf_path=self.conf_path,
            conf=self.conf,
            logger=self.logger
        )

        self.assertIsNotNone(conf)
        self.assertEqual(len(conf), 2)

        unified_conf = conf.unify()

        parameters = unified_conf[Configuration.VALUES]
        errors = unified_conf[Configuration.ERRORS]

        self.assertIn('a', parameters)
        self.assertNotIn('a', errors)
        self.assertEqual(parameters['a'].value, 0)
        self.assertIn('b', parameters)
        self.assertNotIn('b', errors)
        self.assertEqual(parameters['b'].value, 1)
        #print(parameters['c'])
        self.assertIn('c', errors)
        self.assertNotIn('c', parameters)

        # get some conf
        conf = Configuration(self.conf['B'])

        conf = self.manager.get_conf(
            conf_path=self.conf_path,
            conf=conf,
            logger=self.logger
        )

        unified_conf = conf.unify()

        parameters = unified_conf[Configuration.VALUES]
        errors = unified_conf[Configuration.ERRORS]

        self.assertNotIn('a', parameters)
        self.assertNotIn('a', errors)
        self.assertIn('b', parameters)
        self.assertNotIn('b', errors)
        self.assertEqual(parameters['b'].value, 1)
        self.assertNotIn('c', parameters)
        self.assertIn('c', errors)

    def _get_conf_manager(self):
        """Only one method to override by sub tests
        """
        return TestConfDriver()

    def _get_manager_path(self):

        return 'b3j0f.conf.driver.file.test.TestConfDriver'

    def _get_manager(self):

        return TestConfDriver

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['Configuration', 'Category', 'Parameter']


from collections import Iterable

from b3j0f.utils.version import basestring, OrderedDict
from b3j0f.utils.path import lookup


class Configuration(object):
    """Manage conf such as a list of Categories.

    The order of categories permit to ensure param overriding.
    """

    ERRORS = 'ERRORS'  #: category name which contains errors
    VALUES = 'VALUES'  #: category name which contains local parameter values
    FOREIGNS = 'FOREIGN'  #: category name which contains not local parameters

    def __init__(self, *categories):
        """
        :param list categories: categories to configure.
        """

        super(Configuration, self).__init__()

        # set categories
        self.categories = OrderedDict()
        for category in categories:
            self.categories[category.name] = category

    def __iter__(self):

        return iter(self.categories.values())

    def __delitem__(self, category_name):

        del self.categories[category_name]

    def __getitem__(self, category_name):

        return self.categories[category_name]

    def __contains__(self, category_name):

        return category_name in self.categories

    def __len__(self):

        return len(self.categories)

    def __iadd__(self, other):
        """Add categories or conf categories in self.
        """

        # if other is a conf add a copy of all other categories
        if isinstance(other, Configuration):
            for category in other:
                self += category

        else:  # in case of category
            category = self.get(other.name)

            if category is None:
                self.put(other)

            else:
                for param in other:
                    category.put(param)

        return self

    def __repr__(self):

        return 'Configuration({0})'.format(self.categories)

    def get(self, category_name, default=None):

        return self.categories.get(category_name, default)

    def setdefault(self, category_name, category):

        return self.categories.setdefault(category_name, category)

    def put(self, category):
        """Put a category and return the previous one if exist.
        """

        result = self.get(category.name)
        self.categories[category.name] = category
        return result

    def unify(self, copy=False):
        """Get a conf which contains only two categories:

        - VALUES where params are all self params where values are not
            exceptions.contains all values
        - ERRORS where params are all self params where values are
            exceptions

        :param bool copy: copy self params (default False).
        :return: two categories named respectivelly VALUES and ERRORS and
            contain respectivelly self param values and parsing errors.
        :rtype: Configuration
        """

        result = Configuration()

        values = Category(Configuration.VALUES)
        errors = Category(Configuration.ERRORS)
        foreigns = Category(Configuration.FOREIGNS)

        for category in self:
            for param in category:
                if param.value is not None:
                    final_values = values if param.local else foreigns

                    if isinstance(param.value, Exception):
                        to_update = errors
                        to_delete = final_values

                    else:
                        to_update = final_values
                        to_delete = errors

                    to_update.put(param.copy() if copy else param)

                    if param.name in to_delete:
                        del to_delete[param.name]

        result += values
        result += foreigns
        result += errors

        return result

    def get_unified_category(self, name, copy=False):
        """Add a category with input name which takes all params provided
        by other categories.

        :param str name: new category name.

        :param bool copy: copy self params (default False).
        """

        result = Category(name)

        for category in self:
            if not isinstance(category, ParamList):
                for param in category:
                    result.put(param.copy() if copy else param)

        return result

    def add_unified_category(self, name, copy=False, new_content=None):
        """Add a unified category to self and add new_content if not None.
        """

        category = self.get_unified_category(name=name, copy=copy)

        if new_content is not None:
            category += new_content

        self += category

    def add_param_list(self, name, content):
        """Add a list of parameters to self.
        """

        self.categories[name] = content

    def clean(self):
        """Clean this params in setting value to None.
        """

        for category in self:

            category.clean()

    def copy(self, cleaned=False):
        """Copy this Configuration.

        :param bool cleaned: copy this element without parameter values.
        """

        result = Configuration()

        for category in self:
            result.put(category.copy(cleaned=cleaned))

        return result

    def update(self, conf):
        """Update this content with input conf.
        """

        for category in conf:
            category = self.setdefault(
                category.name, category.copy())

            for param in category:
                param = category.setdefault(
                    param.name, param.copy())


class Category(object):
    """Parameter category which contains a dictionary of params.
    """

    def __init__(self, name, *params):
        """
        :param str name: unique in a conf.
        :param list params: Parameters
        """

        super(Category, self).__init__()

        self.name = name
        # set param by names.
        self.params = {}

        for param in params:
            self.params[param.name] = param

    def __iter__(self):

        return iter(self.params.values())

    def __delitem__(self, param_name):

        del self.params[param_name]

    def __getitem__(self, param_name):

        return self.params[param_name]

    def __contains__(self, param_name):

        return param_name in self.params

    def __len__(self):

        return len(self.params)

    def __eq__(self, other):

        return isinstance(other, Category) and other.name == self.name

    def __hash__(self):

        return hash(self.name)

    def __repr__(self):

        return 'Category({0}, {1})'.format(self.name, self.params)

    def __iadd__(self, value):

        if isinstance(value, Category):
            self += value.params.values()

        elif isinstance(value, Iterable):
            for content in value:
                self += content

        elif isinstance(value, Parameter):
            self.put(value)

        else:
            raise Exception('Wrong type to add {0} to {1}. \
Must be a Category, a Parameter or a list of {Parameter, Category}'.format(
                value, self))

        return self

    def setdefault(self, param_name, param):

        return self.params.setdefault(param_name, param)

    def get(self, param_name, default=None):

        return self.params.get(param_name, default)

    def put(self, param):
        """Put a param and return the previous one if exist
        """

        result = self.get(param.name)
        self.params[param.name] = param
        return result

    def clean(self):
        """Clean this params in setting value to None.
        """

        for param in self.params.values():

            param.clean()

    def copy(self, name=None, cleaned=False):
        """Copy this Category.

        :param bool cleaned: copy this element without parameter values.
        """

        if name is None:
            name = self.name

        result = Category(name)

        for param in self:
            _param = param.copy(cleaned=cleaned)
            result.put(_param)

        return result


class Parameter(object):
    """Parameter identified among a category by its name.

    Provide a value (None by default) and a parser (str by default).
    """

    def __init__(
        self, name, parser=None, value=None, critical=False,
        local=True, asitem=None
    ):
        """
        :param str name: unique by category.
        :param value: param value. None if not given.
        :param callable parser: param test deserializer which takes in param
            a str.
        :param bool critical: True if this parameter is critical. A critical
            parameter can require to restart a component for example.
        :param bool local: distinguish local parameters from those found in
            configuration resources.
        :param Category asitem: distinguish parameters from those in a
            ParamList.
        """

        super(Parameter, self).__init__()

        self.name = name
        self._value = value
        self.parser = parser
        self.critical = critical
        self.local = local
        self.asitem = asitem

    def __eq__(self, other):

        return isinstance(other, Parameter) and other.name == self.name

    def __hash__(self):

        return hash(self.name)

    def __repr__(self):

        return 'Parameter({0}, {1}, {2})'.format(
            self.name, self.value, self.parser)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if isinstance(value, basestring) and self.parser is not None:
            # parse value if str and if parser exists
            try:
                self._value = self.parser(value)

            except Exception as e:
                self._value = e

        else:
            self._value = value

    def copy(self, name=None, cleaned=False):

        if name is None:
            name = self.name

        kwargs = {
            "name": name,
            "parser": self.parser
        }
        if not cleaned:
            kwargs["value"] = self.value

        result = Parameter(**kwargs)

        return result

    def clean(self):
        """Clean this param in removing values
        """

        self._value = None

    @staticmethod
    def array(item_type=str):
        """Get an array from an input value where items are separated by ','.
        """

        def split(value):
            return [item_type(v) for v in value.split(',')]

        return split

    @staticmethod
    def bool(value):
        return value == 'True' or value == 'true' or value == '1'

    @staticmethod
    def path(value):
        return lookup(value)


class ParamList(object):
    """Identify the list of parameters in a category.
    """

    def __init__(self, parser=None, *args, **kwargs):
        super(ParamList, self).__init__(*args, **kwargs)

        self.parser = parser



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['Configurables', 'ConfigurableTypes', 'ConfigurableRegistry']


from b3j0f.utils.version import basestring
from b3j0f.utils.path import lookup

from b3j0f.conf.configurable.core import Configurable
from b3j0f.conf.params import Configuration, Category

from inspect import isclass


class Configurables(dict):
    """With a ConfigurableTypes, it is in charge of a ConfigurableRegistry
    sub-configurables.

    When a configurable is trying to be setted, the type is checked related
    to its ConfigurableRegistry ConfigurableTypes.
    """

    def __init__(self, registry, values=None, *args, **kwargs):
        """
        :param ConfigurableRegistry registry: related registry.
        :param dict values: default values if not None (default).
        """

        super(Configurables, self).__init__(*args, **kwargs)

        self.registry = registry

        if values is not None:
            for name, value in values:
                self[name] = value

    def __setitem__(self, name, value):
        """Set a new configurable value.

        :param str name: new configurable name.
        :param value: new configurable value.
        :type value: str (path) or class or instance
        :param args: args for configurable instanciation if value is a path or
            a class.
        :param kwargs: kwargs for configurable instanciation if value is a path
            or a class.
        """

        # get configurable type. Configurable by default
        configurable_type = self.registry._configurable_types.get(
            name, Configurable)

        configurable = value

        # if value is a path
        if isinstance(configurable, basestring):
            # get related python object
            configurable = lookup(configurable)

        # if configurable is a class
        if isclass(configurable) and issubclass(
                configurable, configurable_type):
                # instantiate a new configurable with input args and kwargs
                configurable = configurable()

        # do nothing if configurable is not an instance of configurable_type
        if not isinstance(configurable, configurable_type):
            self.registry.logger.error(
                "Impossible to set {0}:{1}. Not an instance of {2}"
                .format(name, configurable, configurable_type)
            )

        else:
            # update self.configurables
            super(Configurables, self).__setitem__(name, configurable)


class ConfigurableTypes(dict):
    """With a Configurables, it is in charge of a set of configurable type.

    When a new type is setted but the old configurable value does not inherits
    from it, then the old value is removed automatically.
    """

    def __init__(self, registry, values=None, *args, **kwargs):
        """
        :param ConfigurableRegistry registry: related registry.
        :param dict values: default values if not None (default).
        """

        super(ConfigurableTypes, self).__init__(*args, **kwargs)

        self.registry = registry

        if values is not None:

            for name in values:
                value = values[name]
                self[name] = value

    def __setitem__(self, name, value):
        """Set a new configurable type.

        :param str name: new configurable name.
        :param value: new type value.
        :type value: str (path) or class
        """

        configurable_type = value

        # if configurable_type is a path
        if isinstance(configurable_type, basestring):
            # get related python object
            configurable_type = lookup(configurable_type)

        # check if configurable_type is a subclass of Configurable
        if not issubclass(configurable_type, Configurable):
            self.registry.logger.error(
                "Impossible to set configurable type {0}: {1}. Wrong type"
                .format(name, configurable_type)
            )

        else:
            # check if an old value exiss
            if name in self.registry._configurables \
                    and not isinstance(
                        self.registry._configurables[name], configurable_type):
                # if the old value is not an instance of newly type
                self.registry.logger.warning(
                    "Old configurable {0} removed. Not an instance of {1}"
                    .format(name, configurable_type)
                )
                # delete if
                del self.registry._configurables[name]

            # set the new type
            super(ConfigurableTypes, self).__setitem__(name, configurable_type)


class ConfigurableRegistry(Configurable):
    """Manage a set of configurables which are accessibles from self
    configurables.

    Each configurable can be defined in conf parameters where names are like
    {name}_configurable={configurable_path, configurable_class, configurable}.

    And a configurable configuration are in categories {NAME}_CONF.

    Then all sub-configurables are accessibles from item accessors with name as
    item key.
    """

    class Error(Exception):
        """handle ConfigurableRegistry errors"""
        pass

    CONF_PATH = 'configuration/registry.conf'  #: default conf path

    CATEGORY = 'MANAGER'  #: default ConfigurableRegistry category name

    CONFIGURABLE_SUFFIX = '_value'  #: configurable configuration suffix
    CONFIGURABLE_TYPE_SUFFIX = '_type'  #: type config suffix

    def __init__(
        self, configurables=None, configurable_types=None, *args, **kwargs
    ):
        """
        :param dict configurables: dictionary of configurables by name.
        :param dict configurable_types: dictionary of configurable types by
            name.
        """

        super(ConfigurableRegistry, self).__init__(*args, **kwargs)

        self._configurables = Configurables(self, configurables)
        self._configurable_types = ConfigurableTypes(self, configurable_types)

    def _get_category(self):
        """Get category.
        """

        result = Category(ConfigurableRegistry.CATEGORY)

        return result

    def _conf(self, *args, **kwargs):

        result = super(ConfigurableRegistry, self)._conf(*args, **kwargs)

        result.add_unified_category(name=ConfigurableRegistry.CATEGORY)

        return result

    def _get_conf_paths(self, *args, **kwargs):

        result = super(ConfigurableRegistry, self)._get_conf_paths(
            *args, **kwargs
        )

        result.append(ConfigurableRegistry.CONF_PATH)

        return result

    def apply_configuration(
        self,
        conf=None, conf_paths=None, drivers=None, logger=None, override=True,
        *args, **kwargs
    ):

        super(ConfigurableRegistry, self).apply_configuration(
            conf=conf, conf_paths=conf_paths, drivers=drivers, logger=logger,
            override=override,
            *args, **kwargs
        )

        if conf_paths is None:
            conf_paths = self.conf_paths

        if conf is None:
            conf = self.conf

        if drivers is None:
            drivers = self.drivers

        # get self conf path
        conf_path = conf_paths[-1]

        configurables = self._configurables

        # apply configuration to all self configurables
        for name in configurables:
            configurable = configurables[name]
            # add self last conf paths to configurable conf paths
            configurable_conf_paths = list(configurable.conf_paths)
            configurable_conf_paths.append(conf_path)
            # get a copy of configurable configuration
            configurable_configuration = configurable.conf.copy()
            # add a unified category where name is {NAME}_CONF
            category_name = ConfigurableRegistry.get_configurable_category(
                name
            )
            configurable_configuration.add_unified_category(
                name=category_name, copy=True
            )
            # apply configurable configuration
            configurable.apply_configuration(
                conf=configurable_configuration,
                conf_paths=configurable_conf_paths,
                drivers=drivers, logger=logger, override=override
            )

    def _configure(self, unified_conf, *args, **kwargs):

        super(ConfigurableRegistry, self)._configure(
            unified_conf=unified_conf, *args, **kwargs
        )

        foreigns = unified_conf[Configuration.FOREIGNS]

        if foreigns:
            # get len of suffixes in order to extract sub configurable names
            lenconfsuffix = len(ConfigurableRegistry.CONFIGURABLE_SUFFIX)
            lenconftsuffix = len(ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX)

            # for all parameters among foreign parameters
            for parameter in foreigns:
                # if name matches with a configurable name
                if parameter.name.endswith(
                        ConfigurableRegistry.CONFIGURABLE_SUFFIX
                ):
                    name = parameter.name[:-lenconfsuffix]
                    # try update it
                    self._configurables[name] = parameter.value

                # if name matches with a configurable type name
                elif parameter.name.endswith(
                        ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX
                ):
                    name = parameter.name[:-lenconftsuffix]
                    # try to update it
                    self._configurable_types[name] = parameter.value

    def _is_local(self, to_configure, name, *args, **kwargs):

        result = super(ConfigurableRegistry, self)._is_local(
            to_configure, name, *args, **kwargs
        )

        if not result:

            result = (
                name.endswith(ConfigurableRegistry.CONFIGURABLE_SUFFIX)
                or name.endswith(ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX)
            )

        return result

    @property
    def configurables(self):
        """Configurable which manages sub-configurables.
        """

        return self._configurables

    @property
    def configurable_types(self):
        """ConfigurableTypes which manages restriction of sub-configurable
        types.
        """

        return self._configurable_types

    def __contains__(self, name):
        """Redirection to self.configurables.__contains__.
        """

        if name.endswith(ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX):
            return name in self._configurable_types

        return name in self._configurables

    def __getitem__(self, name):
        """Redirection to self.configurables.__getitem__.
        """

        if name.endswith(ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX):
            return self._configurables_types[name]

        return self._configurables[name]

    def __setitem__(self, name, value):
        """Redirection to self.configurables.__setitem__.
        """

        if name.endswith(ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX):
            self._configurables_types[name] = value

        else:
            self._configurables[name] = value

    def __delitem__(self, name):
        """Redirection to self.configurables.__delitem__.
        """

        if name.endswith(ConfigurableRegistry.CONFIGURABLE_TYPE_SUFFIX):
            del self._configurables_types[name]

        else:
            del self._configurables[name]

    @staticmethod
    def get_configurable_category(name):
        """Get generated sub-configurable category name.
        """

        return "{0}_CONF".format(name.upper())

    @staticmethod
    def get_configurable(configurable, *args, **kwargs):
        """Get a configurable instance from a configurable class/path/instance
        and args, kwargs, None otherwise.

        :param configurable: configurable path, class or instance.
        :type configurable: str, class or Configurable
        :return: configurable instance or None if input configurable can not be
            solved such as a configurable.
        """

        result = configurable

        if isinstance(configurable, basestring):
            result = lookup(configurable)

        if issubclass(result, Configurable):
            result = result(*args, **kwargs)

        if not isinstance(result, Configurable):
            result = None

        return result



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['MetaConfigurable', 'Configurable', 'ConfigurableError']


from logging import Formatter, getLogger, FileHandler, Filter

from os.path import join, sep

from inspect import isclass

from b3j0f.utils.version import basestring

from b3j0f.conf.params import Configuration, Category, Parameter
from b3j0f.conf.driver.core import ConfDriver


class MetaConfigurable(type):
    """Meta class for Configurable.
    """

    def __call__(cls, *args, **kwargs):
        """Get a new instance of input cls class, and if instance.auto_conf or
        instance.reconf_once, then call instance.apply_configuration().
        """

        result = type.__call__(cls, *args, **kwargs)

        if result.auto_conf or result.reconf_once:
            # get configuration
            conf = result.conf
            # add a last category which contains args and kwargs as parameters
            if kwargs:
                init_category = Category(Configurable.INIT_CAT)
                for name in kwargs:
                    param = Parameter(name=name, value=kwargs[name])
                    init_category += param
                conf += init_category
            # apply configuration
            result.apply_configuration(conf=conf)

        return result


class ConfigurableError(Exception):
    """Handle Configurable errors
    """
    pass


class Configurable(object):
    """Manages class conf synchronisation with conf resources.
    """

    __metaclass__ = MetaConfigurable

    DEFAULT_DRIVERS = '{0},{1}'.format(
        'b3j0f.conf.driver.file._json.JSONConfDriver',
        'b3j0f.conf.driver.file._ini.INIConfDriver'
    )

    INIT_CAT = 'init_cat'  #: initialization category

    CONF_PATH = 'configuration/configurable.conf'

    CONF = 'CONFIGURATION'
    LOG = 'LOG'

    AUTO_CONF = 'auto_conf'
    RECONF_ONCE = 'reconf_once'
    CONF_PATHS = 'conf_paths'
    DRIVERS = 'drivers'

    LOG_NAME = 'log_name'  #: logger name property name
    LOG_LVL = 'log_lvl'  #: logging level property name
    LOG_PATH = 'log_path'  #: logging path property name
    LOG_DEBUG_FORMAT = 'log_debug_format'  #: debug log format property name
    LOG_INFO_FORMAT = 'log_info_format'  #: info log format property name
    LOG_WARNING_FORMAT = 'log_warning_format'  #: warn log format property name
    LOG_ERROR_FORMAT = 'log_error_format'  #: error log format property name
    LOG_CRITICAL_FORMAT = 'log_critical_format'  #: crit log format property

    DEBUG_FORMAT = "[%(asctime)s] [%(levelname)s] [%(name)s] \
[%(process)d] [%(thread)d] [%(pathname)s] [%(lineno)d] %(message)s"
    INFO_FORMAT = "[%(asctime)s] [%(levelname)s] [%(name)s] %(message)s"
    WARNING_FORMAT = INFO_FORMAT
    ERROR_FORMAT = WARNING_FORMAT
    CRITICAL_FORMAT = ERROR_FORMAT

    def __init__(
        self,
        unified_category=None,
        to_configure=None,
        conf_paths=None, drivers=DEFAULT_DRIVERS,
        auto_conf=True, reconf_once=False,
        log_lvl='INFO', log_name=None, log_path='.',
        log_info_format=INFO_FORMAT,
        log_debug_format=DEBUG_FORMAT, log_warning_format=WARNING_FORMAT,
        log_error_format=ERROR_FORMAT, log_critical_format=CRITICAL_FORMAT
    ):
        """
        :param str unified_category: if not None, used such as a unified
            category.
        :param to_configure: object to reconfigure. Such object may
            implement the methods configure apply_configuration and configure.
        :param conf_paths: conf_paths to parse.
        :type conf_paths: Iterable or str
        :param bool auto_conf: True (default) force auto conf as soon as param
            change.
        :param bool reconf_once: True (default) force auto conf reconf_once as
            soon as param change.
        :param str log_lvl: logging level. Default is INFO.
        :param str log_name: logger name. Default is configurable class lower
            name.
        :param str log_path: logging file path. Default is current directory.
        :param str log_info_format: info logging level format.
        :param str log_debug_format: debug logging level format.
        :param str log_warning_format: warning logging level format.
        :param str log_error_format: error logging level format.
        :param str log_critical_format: critical logging level format.
        """

        super(Configurable, self).__init__()

        self.unified_category = unified_category

        self._to_configure = self if to_configure is None else to_configure

        self.auto_conf = auto_conf
        self.reconf_once = reconf_once

        # set conf files
        self._init_conf_paths(conf_paths)

        # set drivers
        self.drivers = drivers

        # set logging properties
        self._log_lvl = log_lvl
        self._log_path = log_path
        self._log_name = log_name if log_name is not None else \
            type(self).__name__.lower()
        self._log_debug_format = log_debug_format
        self._log_info_format = log_info_format
        self._log_warning_format = log_warning_format
        self._log_error_format = log_error_format
        self._log_critical_format = log_critical_format

        self._logger = self.newLogger()

    @property
    def drivers(self):
        return self._drivers

    @drivers.setter
    def drivers(self, value):
        self._drivers = value

    def newLogger(self):
        """Get a new logger related to self properties.
        """

        result = getLogger(self.log_name)
        result.setLevel(self.log_lvl)

        def setHandler(logger, lvl, path, _format):
            """Set right handler related to input lvl, path and format.

            :param Logger logger: logger on which add an handler.
            :param str lvl: logging level.
            :param str path: file path.
            :param str _format: logging message format.
            """

            class _Filter(Filter):
                """Ensure message will be given for specific lvl
                """
                def filter(self, record):
                    return record.levelname == lvl

            # get the rights formatter and filter to set on a file handler
            handler = FileHandler(path)
            handler.addFilter(_Filter())
            handler.setLevel(lvl)
            formatter = Formatter(_format)
            handler.setFormatter(formatter)

            # if an old handler exist, remove it from logger
            if hasattr(logger, lvl):
                old_handler = getattr(logger, lvl)
                logger.removeHandler(old_handler)

            logger.addHandler(handler)
            setattr(logger, lvl, handler)

        filename = self.log_name.replace('.', sep)
        path = join(self.log_path, '{0}.log'.format(filename))

        setHandler(result, 'DEBUG', path, self.log_debug_format)
        setHandler(result, 'INFO', path, self.log_info_format)
        setHandler(result, 'WARNING', path, self.log_warning_format)
        setHandler(result, 'ERROR', path, self.log_error_format)
        setHandler(result, 'CRITICAL', path, self.log_critical_format)

        return result

    @property
    def conf(self):
        """Get conf with parsers and self property values
        """
        result = self._conf()

        # add a last unified category if asked by self
        if self.unified_category is not None:
            result.add_unified_category(self.unified_category)

        return result

    def _conf(self):
        """Protected method to override in order to specify which conf
        return with parsers and default values
        """

        result = Configuration(
            Category(
                Configurable.CONF,
                Parameter(Configurable.AUTO_CONF, Parameter.bool),
                Parameter(Configurable.DRIVERS),
                Parameter(Configurable.RECONF_ONCE, Parameter.bool),
                Parameter(Configurable.CONF_PATHS, Parameter.array)
            ),
            Category(
                Configurable.LOG,
                Parameter(Configurable.LOG_NAME, critical=True),
                Parameter(Configurable.LOG_PATH, critical=True),
                Parameter(Configurable.LOG_LVL, critical=True),
                Parameter(Configurable.LOG_DEBUG_FORMAT, critical=True),
                Parameter(Configurable.LOG_INFO_FORMAT, critical=True),
                Parameter(Configurable.LOG_WARNING_FORMAT, critical=True),
                Parameter(Configurable.LOG_ERROR_FORMAT, critical=True),
                Parameter(Configurable.LOG_CRITICAL_FORMAT, critical=True)
            )
        )

        return result

    @property
    def log_debug_format(self):

        return self._log_debug_format

    @log_debug_format.setter
    def log_debug_format(self, value):

        self._log_debug_format = value
        self._logger = self.newLogger()

    @property
    def log_info_format(self):

        return self._log_info_format

    @log_info_format.setter
    def log_info_format(self, value):

        self._log_info_format = value
        self._logger = self.newLogger()

    @property
    def log_warning_format(self):

        return self._log_warning_format

    @log_warning_format.setter
    def log_warning_format(self, value):

        self._log_warning_format = value
        self._logger = self.newLogger()

    @property
    def log_error_format(self):

        return self._log_error_format

    @log_error_format.setter
    def log_error_format(self, value):

        self._log_error_format = value
        self._logger = self.newLogger()

    @property
    def log_critical_format(self):

        return self._log_critical_format

    @log_critical_format.setter
    def log_critical_format(self, value):

        self._log_critical_format = value
        self._logger = self.newLogger()

    @property
    def log_name(self):

        return self._log_name

    @log_name.setter
    def log_name(self, value):

        self._log_name = value
        self._logger = self.newLogger()

    @property
    def log_path(self):

        return self._log_path

    @log_path.setter
    def log_path(self, value):

        self._log_path = value
        self._logger = self.newLogger()

    @property
    def log_lvl(self):
        """Get this logger lvl.

        :return: self logger lvl
        :rtype: str
        """

        return self._log_lvl

    @log_lvl.setter
    def log_lvl(self, value):
        """Change of logging level.

        :param str value: new log_lvl to set up.
        """

        self._log_lvl = value
        self._logger.setLevel(self._log_lvl)

    @property
    def logger(self):

        return self._logger

    @logger.setter
    def logger(self, value):

        self._logger = value

    @property
    def conf_paths(self):
        """Get all type conf files and user files.

        :return: self conf files
        :rtype: tuple
        """

        if not hasattr(self, '_conf_paths'):
            self._conf_paths = []

        result = self._conf_paths

        return result

    @conf_paths.setter
    def conf_paths(self, value):
        """Change of conf_paths in adding it in watching list.

        .. TODO:: add watchers here
        """

        #from b3j0f.conf.watcher import add_configurable,\
        #    remove_configurable

        # remove previous watching
        #remove_configurable(self)
        self._conf_paths = tuple(value)
        # add new watching
        #add_configurable(self)

    @property
    def auto_conf(self):
        return self._auto_conf

    @auto_conf.setter
    def auto_conf(self, value):
        self._auto_conf = value

    @property
    def reconf_once(self):
        return self._reconf_once

    @reconf_once.setter
    def reconf_once(self, value):
        self._reconf_once = value

    def apply_configuration(
        self, conf=None, conf_paths=None, drivers=None, logger=None,
        override=True, to_configure=None
    ):
        """Apply conf on a destination in 5 phases:

        1. identify the right driver to use with conf_paths to parse.
        2. for all conf_paths, get conf which match
            with input conf.
        3. apply parsing rules on conf_path params.
        4. put values and parsing errors in two different dictionaries.
        5. returns both dictionaries of param values and errors.

        :param Configuration conf: conf from where get conf
        :param conf_paths: conf files to parse. If
            conf_paths is a str, it is automatically putted into a list
        :type conf_paths: list of str
        :param bool override: if True (by default), override self configuration
        :param to_configure: object to configure. self by default.
        """

        conf = self.get_conf(
            conf=conf, conf_paths=conf_paths, logger=logger,
            drivers=drivers, override=override
        )

        self.configure(conf=conf, to_configure=to_configure)

    def get_conf(
        self,
        conf=None, conf_paths=None, drivers=None, logger=None,
        override=True
    ):
        """Get a dictionary of params by name from conf,
        conf_paths and drivers

        :param Configuration conf: conf to update. If None, use self.conf
        :param conf_paths: list of conf files. If None, use self.conf_paths
        :type conf_paths: list of str
        :param Logger logger: logger to use for logging info/error messages.
            If None, use self.logger
        :param list drivers: ConfDriver to use. If None, use self
            drivers.
        :param bool override: if True (by default), override self configuration
        :param to_configure: object to configure. self by default.
        """

        # start to initialize input params
        if logger is None:
            logger = self.logger

        if conf is None:
            conf = self.conf

        if conf_paths is None:
            conf_paths = self._conf_paths

        if isinstance(conf_paths, basestring):
            conf_paths = [conf_paths]

        if drivers is None:
            drivers = self.drivers

        # iterate on all conf_paths
        for conf_path in conf_paths:

            conf_driver = self._get_driver(
                conf_path=conf_path, logger=logger, drivers=drivers
            )

            # if a config_resource is not None
            if conf_driver is not None:

                conf = conf_driver.get_conf(
                    conf=conf, logger=logger,
                    conf_path=conf_path, override=override
                )

            else:
                # if no conf_driver, display a warning log message
                self.logger.warning(
                    'No driver found among {0} for processing {1}'.format(
                        drivers, conf_path
                    )
                )

        return conf

    def set_conf(self, conf_path, conf, driver=None, logger=None):
        """Set params on input conf_path.

        :param str conf_paths: conf_path to udate with params
        :param dict parameter_by_categories: (dict(str: dict(str: object))
        :param Logger logger: logger to use to set params.
        """

        result = None

        if logger is None:
            logger = self.logger

        # first get content of input conf_path
        prev_driver = self._get_driver(
            conf_path=conf_path,
            logger=logger,
            drivers=self.drivers
        )

        if prev_driver is not None:
            prev_conf = prev_driver.get_conf(
                conf_path=conf_path, logger=logger
            )

        # try to find a good driver if driver is None
        if driver is None:
            driver = self._get_driver(
                conf_path=conf_path,
                logger=logger,
                drivers=self.drivers
            )

        elif isclass(driver):
            driver = driver()

        else:
            driver = self._get_driver(
                conf_path=None,
                logger=logger,
                drivers=driver)

        # if prev driver is not the new driver
        if prev_conf is not None \
                and type(driver) is not type(prev_driver):
            # update prev_conf with input conf
            prev_conf.update(conf)
            conf = prev_conf

        if driver is not None:
            driver.set_conf(
                conf_path=conf_path,
                conf=conf,
                logger=logger
            )

        else:
            self.logger.error(
                'No ConfDriver found for conf resource {0}'.format(
                    conf_path
                )
            )

        return result

    def configure(self, conf, logger=None, to_configure=None):
        """Update self properties with input params only if:
        - self.configure is True
        - self.auto_conf is True
        - param conf 'configure' is True
        - param conf 'auto_conf' is True

        This method may not be overriden. see _configure instead

        :param Configuration conf: object from where get paramters
        :param to_configure: object to configure. self if equals None.
        """

        if logger is None:
            logger = self.logger

        unified_conf = conf.unify()

        values = unified_conf[Configuration.VALUES]

        # set configure
        reconf_once = values.get(Configurable.RECONF_ONCE)

        if reconf_once is not None:
            self.reconf_once = reconf_once.value

        # set auto_conf
        auto_conf_parameter = values.get(Configurable.AUTO_CONF)

        if auto_conf_parameter is not None:
            self.auto_conf = auto_conf_parameter.value

        if self.reconf_once or self.auto_conf:
            self._configure(unified_conf=unified_conf, logger=logger)
            # when conf succeed, deactive reconf_once
            self.reconf_once = False

    def _is_local(self, to_configure, name):
        """True iif input name parameter can be handled by to_configure.
        """

        return hasattr(to_configure, name)

    def _configure(self, unified_conf, logger=None, to_configure=None):
        """Configure this class with input conf only if auto_conf or
        configure is true.

        This method should be overriden for specific conf

        :param Configuration unified_conf: Configuration with two categories
            VALUES and ERRORS
        :param bool configure: if True, force full self conf
        :param to_configure: object to configure. self if equals None.
        """

        if to_configure is None:
            to_configure = self

        values = [p for p in unified_conf[Configuration.VALUES]]
        foreigns = [p for p in unified_conf[Configuration.FOREIGNS]]

        criticals = []  # list of critical parameters

        for parameter in values + foreigns:
            name = parameter.name
            if not parameter.asitem:
                # if parameter is local, to_configure must a related name
                if self._is_local(to_configure, name):
                    if hasattr(to_configure, name):
                        param_value = parameter.value

                        # in case of a critical parameter
                        if parameter.critical:
                            # check if current value is not the same as new val
                            value = getattr(to_configure, name)

                            if value != parameter.value:
                                # add it to list of criticals
                                criticals.append(parameter)
                                # set private name
                                privname = '_%s' % name

                                # if private name exists
                                if hasattr(to_configure, privname):
                                    # change of value of private name
                                    setattr(
                                        to_configure, privname, param_value)

                                else:  # update public value
                                    setattr(to_configure, name, param_value)

                        else:  # change public value
                            setattr(to_configure, name, param_value)

                else:  # else log the warning
                    message = 'Param {0} is not bound to an attribute in {1}'
                    self.logger.warning(message.format(name, to_configure))

            else:
                value = getattr(to_configure, parameter.asitem.name)
                param_value = parameter.value
                param_name = parameter.name

                if param_name not in value or param_value != value[param_name]:
                    criticals.append(parameter)

                value[parameter.name] = parameter.value

        # if criticals
        if criticals:
            self.restart(to_configure=to_configure, criticals=criticals)

    def restart(self, criticals, to_configure=None):
        """Restart a configurable object with critical parameters.

        :param to_configure: object to configure with critical parameters
        :param tuple criticals: list of critical parameters
        """

        if to_configure is None:
            to_configure = self

        if self._is_critical_category(Configurable.LOG, criticals):
            self._logger = self.newLogger()
            to_configure.logger = self.logger

    def _is_critical_category(self, category, criticals):
        """Check if input category parameters are among criticals.
        """

        result = False

        properties = (param.name for param in self.conf[category])

        # if property is among criticals
        for prop in properties:
            for critical in criticals:
                if critical.name == prop:
                    result = True
                    break

        return result

    def _init_conf_paths(self, conf_paths):

        self.conf_paths = self._get_conf_paths() \
            if conf_paths is None else conf_paths

    def _get_conf_paths(self):

        result = [Configurable.CONF_PATH]

        return result

    @staticmethod
    def _get_driver(conf_path, drivers, logger):
        """Get the first driver able to handle input conf_path.
        None if no driver is able to handle input conf_path.

        :return: first ConfDriver able to handle conf_path.
        :rtype: ConfDriver
        """

        result = None

        for driver in drivers.split(','):
            driver = ConfDriver.get_driver(driver)
            driver = driver()

            handle = conf_path is None \
                or driver.handle(conf_path=conf_path, logger=logger)

            if handle:
                result = driver
                break

        return result



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

__all__ = ['conf_paths', 'add_category']


from b3j0f.conf.configurable.core import Configurable
from b3j0f.conf.params import Category, ParamList


def conf_paths(*conf_paths):
    """Configurable decorator which adds conf_path paths to a Configurable.

    :param paths: conf resource pathes to add to a Configurable.
    :type paths: list of str

    Example:
    >>> conf_paths('test0', 'test1')(Configurable)().conf_paths[:-2]
    ['test0', 'test1']
    """

    def add_conf_paths(cls):
        # add _get_conf_paths method to configurable classes
        if issubclass(cls, Configurable):

            def _get_conf_paths(self, *args, **kwargs):
                # get super result and append conf_paths
                result = super(cls, self)._get_conf_paths()
                result += conf_paths

                return result

            cls._get_conf_paths = _get_conf_paths

        else:
            raise Configurable.Error(
                "class {0} is not a Configurable class".format(cls))

        return cls

    return add_conf_paths


def add_category(name, unified=True, content=None):
    """Add a category to a configurable configuration.

    :param str name: category name.
    :param bool unified: if True (by default), the new category is unified from
        previous conf.
    :param content: category or list of parameters to add to the new category.
    :type content: Category or list(Parameter)
    """

    def add_conf(cls):

        if issubclass(cls, Configurable):

            def _conf(self, *args, **kwargs):

                result = super(cls, self)._conf(*args, **kwargs)

                if unified:
                    result.add_unified_category(name=name, new_content=content)
                else:
                    category = Category(name=name)
                    if content is not None:
                        category += content
                    result += category

                return result

            cls._conf = _conf

        else:
            raise Configurable.Error(
                "class {0} is not a Configurable class".format(cls))

        return cls

    return add_conf


def add_config(config, unified=True):
    """Add multiple categories to a configurable configuration.

    :param dict config: dict where keys are catogories names, and values
        categories content.
    :param bool unified: if True (by default), the new category is unified from
        previous conf.
    """

    def _add_unified(result, name, content):
        result.add_unified_category(name=name, new_content=content)
        return result

    def _add_not_unified(result, name, content):
        category = Category(name=name)

        if content is not None:
            category += content

        result += category

        return result

    def _add_category(result, name, content, unified):
        if isinstance(content, ParamList):
            result.add_param_list(name=name, content=content)

        else:
            if unified:
                result = _add_unified(result, name, content)

            else:
                result = _add_not_unified(result, name, content)

        return result

    def _conf(self, *args, **kwargs):
        result = super(type(self), self)._conf(*args, **kwargs)

        for name in config.keys():
            result = _add_category(result, name, config[name], unified)

        return result

    def add_conf(cls):
        if issubclass(cls, Configurable):
            cls._conf = _conf

        else:
            raise Configurable.Error(
                'class {0} is not a Configurable class'.format(cls)
            )

        return cls

    return add_conf



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

from setuptools import setup, find_packages

from os.path import abspath, dirname, join

# get setup directory abspath
_path = dirname(abspath(__file__))

# get long description
with open(join(_path, 'README.rst')) as f:
    desc = f.read()

keywords = [
    'conf', 'configuration', 'configurable', 'class', 'ini', 'json', 'xml',
    'tools', 'property', 'dynamic', 'reflection', 'reflect', 'runtime'
]

dependencies = ['b3j0f.utils']

description = 'python class configuration tools useful in python projects.'

setup(
    name='b3j0f.conf',
    version='0.1.4',
    packages=find_packages(exclude=['test.*', '*.test.*']),
    author='b3j0f',
    author_email='jlabejof@yahoo.fr',
    install_requires=dependencies,
    description=description,
    long_description=desc,
    include_package_data=True,
    url='https://github.com/b3j0f/conf/',
    license='MIT License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: French',
        'Operating System :: OS Independent',
        'Topic :: Utilities',
        'Topic :: Software Development',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ],
    test_suite='b3j0f',
    keywords=keywords
)
